@@ -69,7 +69,7 @@ bool Cleanup::worksFor( FileInfo *item ) const
     if ( ! _active || ! item )
 	return false;
 
-    if	( item->isDotEntry() )	return worksForDotEntry();
+    if	( item->isPseudoDir() )	return worksForDotEntry();
     if	( item->isDir() )	return worksForDir();
 
     return worksForFile();
@@ -148,11 +148,11 @@ void Cleanup::executeRecursive( FileInfo *item, OutputWindow * outputWindow )
 
 const QString Cleanup::itemDir( const FileInfo *item ) const
 {
-    QString dir = item->url();
+    QString dir = item->path();
 
-    if ( ! item->isDir() && ! item->isDotEntry() )
+    if ( ! item->isDir() && ! item->isPseudoDir() )
     {
-	dir.replace ( QRegExp ( "/[^/]*$" ), "" );
+	dir.replace( QRegExp ( "/[^/]*$" ), "" );
     }
 
     return dir;
@@ -183,11 +183,11 @@ QString Cleanup::expandVariables( const FileInfo * item,
     QString dirName = "";
 
     if ( item->isDir() )
-	dirName = item->url();
+	dirName = item->path();
     else if ( item->parent() )
-	dirName = item->parent()->url();
+	dirName = item->parent()->path();
 
-    expanded.replace( "%p", quoted( escaped( item->url()  ) ) );
+    expanded.replace( "%p", quoted( escaped( item->path() ) ) );
     expanded.replace( "%n", quoted( escaped( item->name() ) ) );
 
     if ( ! dirName.isEmpty() )
