@@ -45,7 +45,7 @@ DirInfo::DirInfo( const QString & filenameWithoutPath,
     init();
     ensureDotEntry();
 
-    _directChildrenCount++;
+    _directChildrenCount++;     // One for the newly created dot entry
 }
 
 
@@ -65,7 +65,7 @@ DirInfo::DirInfo( DirTree *	  tree,
     init();
     ensureDotEntry();
 
-    _directChildrenCount++;
+    _directChildrenCount++;     // One for the newly created dot entry
 }
 
 
@@ -90,7 +90,9 @@ void DirInfo::init()
     _totalIgnoredItems	 = 0;
     _totalUnignoredItems = 0;
     _directChildrenCount = 0;
+    _errSubDirCount      = 0;
     _latestMtime	 = _mtime;
+    _oldestFileMtime     = 0;
     _readState		 = DirQueued;
     _sortedChildren	 = 0;
     _lastSortCol	 = UndefinedCol;
@@ -228,7 +230,9 @@ void DirInfo::recalc()
     _totalIgnoredItems	 = 0;
     _totalUnignoredItems = 0;
     _directChildrenCount = 0;
+    _errSubDirCount      = 0;
     _latestMtime	 = _mtime;
+    _oldestFileMtime     = 0;
 
     FileInfoIterator it( this );
 
@@ -239,13 +243,19 @@ void DirInfo::recalc()
 	_totalBlocks	     += (*it)->totalBlocks();
 	_totalItems	     += (*it)->totalItems() + 1;
 	_totalSubDirs	     += (*it)->totalSubDirs();
+        _errSubDirCount      += (*it)->errSubDirCount();
 	_totalFiles	     += (*it)->totalFiles();
 	_totalIgnoredItems   += (*it)->totalIgnoredItems();
 	_totalUnignoredItems += (*it)->totalUnignoredItems();
 
 	if ( (*it)->isDir() )
+        {
 	    _totalSubDirs++;
 
+            if ( (*it)->readError() )
+                _errSubDirCount++;
+        }
+
 	if ( (*it)->isFile() )
 	    _totalFiles++;
 
@@ -262,11 +272,25 @@ void DirInfo::recalc()
 	if ( childLatestMtime > _latestMtime )
 	    _latestMtime = childLatestMtime;
 
+        time_t childOldestFileMTime = (*it)->oldestFileMtime();
+
+        if ( childOldestFileMTime > 0 )
+        {
+            if ( _oldestFileMtime == 0 ||
+                 childOldestFileMTime < _oldestFileMtime )
+            {
+                _oldestFileMtime = childOldestFileMTime;
+            }
+        }
+
 	++it;
     }
 
     if ( _attic )
+    {
 	_totalIgnoredItems += _attic->totalIgnoredItems();
+        _errSubDirCount    += _attic->errSubDirCount();
+    }
 
     _summaryDirty = false;
 }
@@ -359,6 +383,15 @@ time_t DirInfo::latestMtime()
 }
 
 
+time_t DirInfo::oldestFileMtime()
+{
+    if ( _summaryDirty )
+	recalc();
+
+    return _oldestFileMtime;
+}
+
+
 int DirInfo::directChildrenCount()
 {
     if ( _summaryDirty )
@@ -389,6 +422,15 @@ int DirInfo::countDirectChildren()
 }
 
 
+int DirInfo::errSubDirCount()
+{
+    if ( _summaryDirty )
+	recalc();
+
+    return _errSubDirCount;
+}
+
+
 bool DirInfo::isFinished()
 {
     return ! isBusy();
@@ -530,6 +572,17 @@ void DirInfo::childAdded( FileInfo * newChild )
 
 	    if ( newChild->mtime() > _latestMtime )
 		_latestMtime = newChild->mtime();
+
+            time_t childOldestFileMTime = newChild->oldestFileMtime();
+
+            if ( childOldestFileMTime > 0 && newChild->isFile() )
+            {
+                if ( _oldestFileMtime == 0 ||
+                     childOldestFileMTime < _oldestFileMtime )
+                {
+                    _oldestFileMtime = childOldestFileMTime;
+                }
+            }
 	}
 	else
 	{
@@ -645,24 +698,27 @@ void DirInfo::readJobAdded()
 }
 
 
-void DirInfo::readJobFinished()
+void DirInfo::readJobFinished( DirInfo * dir )
 {
     _pendingReadJobs--;
 
     if ( _lastSortCol == ReadJobsCol )
 	dropSortCache();
 
+    if ( dir && dir != this && dir->readError() )
+        _errSubDirCount++;
+
     if ( _parent )
-	_parent->readJobFinished();
+	_parent->readJobFinished( dir );
 }
 
 
-void DirInfo::readJobAborted()
+void DirInfo::readJobAborted( DirInfo * dir )
 {
     _readState = DirAborted;
 
     if ( _parent )
-	_parent->readJobAborted();
+	_parent->readJobAborted( dir );
 }
 
 
@@ -672,6 +728,54 @@ DirReadState DirInfo::readState() const
 }
 
 
+bool DirInfo::readError() const
+{
+    switch ( _readState )
+    {
+        case DirError:
+        case DirPermissionDenied:
+            return true;
+
+	case DirQueued:
+	case DirReading:
+	case DirFinished:
+	case DirOnRequestOnly:
+	case DirCached:
+        case DirAborted:
+            return false;
+
+        // No 'default' branch so the compiler can catch unhandled enum values
+    }
+
+    return false;
+}
+
+
+QString DirInfo::sizePrefix() const
+{
+    switch ( _readState )
+    {
+	case DirQueued:
+	case DirReading:
+	case DirOnRequestOnly:
+            return "";
+
+        case DirError:
+        case DirAborted:
+        case DirPermissionDenied:
+            return ">";
+
+	case DirFinished:
+	case DirCached:
+            return _errSubDirCount > 0 ? ">" : "";
+
+        // No 'default' branch so the compiler can catch unhandled enum values
+    }
+
+    return "";
+}
+
+
 void DirInfo::finalizeLocal()
 {
     // logDebug() << this << endl;
