@@ -143,6 +143,17 @@ namespace QDirStat
 	 **/
 	virtual int directChildrenCount() Q_DECL_OVERRIDE;
 
+        /**
+         * Returns the number of subdirectories below this item that could not
+         * be read (typically due to insufficient permissions).
+         *
+         * Notice that this does NOT include this item if it is a directory
+         * that could not be read.
+	 *
+	 * Reimplemented - inherited from FileInfo.
+         **/
+        virtual int errSubDirCount() Q_DECL_OVERRIDE;
+
 	/**
 	 * Returns the latest modification time of this subtree.
 	 *
@@ -150,6 +161,13 @@ namespace QDirStat
 	 **/
 	virtual time_t latestMtime() Q_DECL_OVERRIDE;
 
+	/**
+	 * Returns the oldest modification time of any file in this subtree.
+         *
+	 * Reimplemented - inherited from FileInfo.
+	 **/
+        virtual time_t oldestFileMtime() Q_DECL_OVERRIDE;
+
 	/**
 	 * Returns 'true' if this had been excluded while reading.
 	 **/
@@ -333,16 +351,15 @@ namespace QDirStat
 	void readJobAdded();
 
 	/**
-	 * Notification of a finished directory read job somewhere in the
-	 * subtree.
+	 * Notification of a finished directory read job for 'dir'.
+         * This is cascaded upward in the tree.
 	 **/
-	void readJobFinished();
+	void readJobFinished( DirInfo * dir );
 
 	/**
-	 * Notification of an aborted directory read job somewhere in the
-	 * subtree.
+	 * Notification of an aborted directory read job for 'dir'.
 	 **/
-	void readJobAborted();
+	void readJobAborted( DirInfo * dir );
 
 	/**
 	 * Finalize this directory level after reading it is completed. This
@@ -362,16 +379,41 @@ namespace QDirStat
 	/**
 	 * Get the current state of the directory reading process:
 	 *
-	 *    DirQueued	   waiting in the directory read queue
-	 *    DirReading   reading in progress
-	 *    DirFinished  reading finished and OK
-	 *    DirAborted   reading aborted upon user request
-	 *    DirError	   error while reading
+	 *    DirQueued	            waiting in the directory read queue
+	 *    DirReading            reading in progress
+	 *    DirFinished           reading finished and OK
+	 *    DirAborted            reading aborted upon user request
+	 *    DirError	            error while reading
+         *    DirPermissionDenied   insufficient permissions
 	 *
 	 * Reimplemented - inherited from FileInfo.
 	 **/
 	virtual DirReadState readState() const Q_DECL_OVERRIDE;
 
+        /**
+         * Check if readState() is anything that indicates an error reading the
+         * directory. This returns 'true' for DirError or DirPermissionDenied,
+         * 'false' otherwise.
+         *
+	 * Reimplemented - inherited from FileInfo.
+         **/
+        virtual bool readError() const Q_DECL_OVERRIDE;
+
+        /**
+         * Return a prefix for the total size (and similar accumulated fields)
+         * of this item: ">" if there might be more, i.e. if a subdirectory
+         * could not be read or if reading was aborted, an empty string
+         * otherwise.
+         *
+         * Notice that this implementation also returns an empty string as long
+         * as this subtree is busy, i.e. reading is not finished: The ">"
+         * prefix should be something special to catch the user's attention,
+         * not something omnipresent that is commonly ignored.
+         *
+         * Reimplemented from FileInfo.
+         **/
+        virtual QString sizePrefix() const Q_DECL_OVERRIDE;
+
 	/**
 	 * Set the state of the directory reading process.
 	 * See readState() for details.
@@ -540,7 +582,9 @@ namespace QDirStat
 	int		_totalIgnoredItems;
 	int		_totalUnignoredItems;
 	int		_directChildrenCount;
+        int             _errSubDirCount;
 	time_t		_latestMtime;
+        time_t          _oldestFileMtime;
 
 	FileInfoList *	_sortedChildren;
 	DataColumn	_lastSortCol;
