@@ -43,7 +43,7 @@ DirReadJob::DirReadJob( DirTree * tree,
 DirReadJob::~DirReadJob()
 {
     if ( _dir )
-	_dir->readJobFinished();
+	_dir->readJobFinished( _dir );
 }
 
 
@@ -98,7 +98,20 @@ bool DirReadJob::crossingFileSystems( DirInfo * parent, DirInfo * child )
 	return false;
 
     QString childDevice	 = device( child );
-    QString parentDevice = device( parent->findNearestMountPoint() );;
+    QString parentDevice = device( parent->findNearestMountPoint() );
+
+    if ( childDevice.isEmpty() && parent->readState() == DirCached )
+    {
+        // A DirInfo from a cache file always has device number 0, so the
+        // initial check failed because of that. The child might still be a
+        // mount point, but since that path was not found in the list of known
+        // mount points from /proc/mounts or /etc/mtab, we assume that this was
+        // not the case.
+        //
+        // See also https://github.com/shundhammer/qdirstat/issues/114
+
+        return false;
+    }
 
     if ( parentDevice.isEmpty() )
 	parentDevice = _tree->device();
@@ -110,9 +123,9 @@ bool DirReadJob::crossingFileSystems( DirInfo * parent, DirInfo * child )
 
     if ( crossing )
     {
-	    logInfo() << "File system boundary at mount point " << child
-		      << " on device " << ( childDevice.isEmpty() ? "<unknown>" : childDevice )
-		      << endl;
+	logInfo() << "File system boundary at mount point " << child
+		  << " on device " << ( childDevice.isEmpty() ? "<unknown>" : childDevice )
+		  << endl;
     }
     else
     {
@@ -173,7 +186,7 @@ void LocalDirReadJob::startReading()
     {
 	ok = false;
 	logWarning() << "No permission to read directory " << _dirName << endl;
-	finishReading( _dir, DirError );
+	finishReading( _dir, DirPermissionDenied );
     }
 
     if ( ok )
@@ -676,13 +689,13 @@ void DirReadJobQueue::abort()
     foreach ( DirReadJob * job, _queue )
     {
 	if ( job->dir() )
-	    job->dir()->readJobAborted();
+	    job->dir()->readJobAborted( job->dir() );
     }
 
     foreach ( DirReadJob * job, _blocked )
     {
 	if ( job->dir() )
-	    job->dir()->readJobAborted();
+	    job->dir()->readJobAborted( job->dir() );
     }
 
     clear();
