@@ -14,13 +14,13 @@
 #include <dirent.h>
 #include <QTimer>
 
+#include "FileInfo.h"
 #include "Logger.h"
 
 
 namespace QDirStat
 {
     // Forward declarations
-    class FileInfo;
     class DirInfo;
     class DirTree;
     class CacheReader;
@@ -35,17 +35,15 @@ namespace QDirStat
      *
      * Objects of this kind are transient by nature: They live only as long as
      * the job is queued or executed. When it is done, the data is contained in
-     * the corresponding @ref DirInfo subtree of the corresponding @ref
-     * DirTree.
+     * the corresponding DirInfo subtree of the corresponding DirTree.
      *
-     * For each entry automatically a @ref FileInfo or @ref DirInfo will be
-     * created and added to the parent @ref DirInfo. For each directory a new
-     * @ref DirReadJob will be created and added to the @ref DirTree 's job
-     * queue.
+     * For each entry automatically a FileInfo or DirInfo will be created and
+     * added to the parent DirInfo. For each directory a new DirReadJob will be
+     * created and added to the DirTree 's job queue.
      *
-     * Notice: This class contains pure virtuals - you cannot use it
-     * directly. Derive your own class from it or use one of
-     * @ref LocalDirReadJob or @ref CacheReadJob.
+     * Notice: This class contains pure virtuals - you cannot use it directly.
+     * Derive your own class from it or use one of LocalDirReadJob or
+     * CacheReadJob.
      *
      * @short Abstract base class for directory reading.
      **/
@@ -76,20 +74,20 @@ namespace QDirStat
 	virtual void read();
 
 	/**
-	 * Returns the corresponding @ref DirInfo item.
+	 * Returns the corresponding DirInfo item.
 	 * Caution: This may be 0.
 	 **/
 	virtual DirInfo * dir() { return _dir; }
 
 	/**
-	 * Set the corresponding @ref DirInfo item.
+	 * Set the corresponding DirInfo item.
 	 **/
 	virtual void setDir( DirInfo * dir );
 
-        /**
-         * Return the corresponding DirTree.
-         **/
-        DirTree * tree() const { return _tree; }
+	/**
+	 * Return the corresponding DirTree.
+	 **/
+	DirTree * tree() const { return _tree; }
 
 	/**
 	 * Return the job queue this job is in or 0 if it isn't queued.
@@ -115,8 +113,8 @@ namespace QDirStat
 	 * Notification that a new child has been added.
 	 *
 	 * Derived classes are required to call this whenever a new child is
-	 * added so this notification can be passed up to the @ref DirTree
-	 * which in turn emits a corresponding signal.
+	 * added so this notification can be passed up to the DirTree which in
+	 * turn emits a corresponding signal.
 	 **/
 	void childAdded( FileInfo *newChild );
 
@@ -124,8 +122,8 @@ namespace QDirStat
 	 * Notification that a child is about to be deleted.
 	 *
 	 * Derived classes are required to call this just before a child is
-	 * deleted so this notification can be passed up to the @ref DirTree
-	 * which in turn emits a corresponding signal.
+	 * deleted so this notification can be passed up to the DirTree which
+	 * in turn emits a corresponding signal.
 	 *
 	 * Derived classes are not required to handle child deletion at all,
 	 * but if they do, calling this method is required.
@@ -138,17 +136,17 @@ namespace QDirStat
 	 **/
 	void finished();
 
-        /**
-         * Check if going from 'parent' to 'child' would cross a file system
-         * boundary. This take Btrfs subvolumes into account.
-         **/
-        bool crossingFileSystems( DirInfo * parent, DirInfo * child );
+	/**
+	 * Check if going from 'parent' to 'child' would cross a file system
+	 * boundary. This take Btrfs subvolumes into account.
+	 **/
+	bool crossingFileSystems( DirInfo * parent, DirInfo * child );
 
-        /**
-         * Return the device name where 'dir' is on if it's a mount point.
-         * This uses MountPoints which reads /proc/mounts.
-         **/
-        QString device( const DirInfo * dir ) const;
+	/**
+	 * Return the device name where 'dir' is on if it's a mount point.
+	 * This uses MountPoints which reads /proc/mounts.
+	 **/
+	QString device( const DirInfo * dir ) const;
 
 
 	DirTree *	   _tree;
@@ -184,7 +182,7 @@ namespace QDirStat
 
 
     /**
-     * Impementation of the abstract @ref DirReadJob class that reads a local
+     * Impementation of the abstract DirReadJob class that reads a local
      * directory.
      *
      * This will use lstat() system calls rather than KDE's network transparent
@@ -210,33 +208,112 @@ namespace QDirStat
 	virtual ~LocalDirReadJob();
 
 	/**
-	 * Obtain information about the URL specified and create a new @ref
-	 * FileInfo or a @ref DirInfo (whatever is appropriate) from that
-	 * information. Use @ref FileInfo::isDirInfo() to find out which.
-	 * Returns 0 if such information cannot be obtained (i.e. the
-	 * appropriate stat() call fails).
+	 * Obtain information about the URL specified and create a new FileInfo
+	 * or a DirInfo (whatever is appropriate) from that information. Use
+	 * FileInfo::isDirInfo() to find out which.
+	 *
+	 * If the underlying syscall fails, this throws a SysCallException if
+	 * 'doThrow' is 'true', and it just returns 0 if it is 'false'.
 	 **/
 	static FileInfo * stat( const QString & url,
 				DirTree	      * tree,
-				DirInfo	      * parent = 0 );
+				DirInfo	      * parent	= 0,
+				bool		doThrow = true );
+
+	/**
+	 * Return 'true' if any exclude rules matching against any direct file
+	 * child should be applied. This is generally useful only for
+	 * second-level read jobs, not for the starting point of a directory
+	 * scan, so it is easily possible to continue reading at an excluded
+	 * directory.
+	 *
+	 * The default is 'false'.
+	 **/
+	bool applyFileChildExcludeRules() const
+	    { return _applyFileChildExcludeRules; }
+
+	/**
+	 * Set the applyFileChildExcludeRules flag.
+	 **/
+	void setApplyFileChildExcludeRules( bool val )
+	    { _applyFileChildExcludeRules = val; }
 
     protected:
 
 	/**
 	 * Read the directory. Prior to this nothing happens.
 	 *
-	 * Inherited and reimplemented from @ref DirReadJob.
+	 * Inherited and reimplemented from DirReadJob.
 	 **/
 	virtual void startReading();
 
 	/**
-	 * Finish reading the directory: Send signals and finalize the
-	 * directory (clean up dot entries etc.).
+	 * Finish reading the directory: Set the specified read state, send
+	 * signals and finalize the directory (clean up dot entries etc.).
 	 **/
-	void finishReading( DirInfo * dir );
+	void finishReading( DirInfo * dir, DirReadState readState );
+
+	/**
+	 * Process one subdirectory entry.
+	 **/
+	void processSubDir( const QString & entryName,
+			    DirInfo	  * subDir    );
+
+	/**
+	 * Return 'true' if 'entryName' matches an exclude rule of the
+	 * ExcludeRule singleton or a temporary exclude rule of the DirTree.
+	 **/
+	bool matchesExcludeRule( const QString & entryName ) const;
+
+	/**
+	 * Return 'true' if 'entryName' should be ignored.
+	 **/
+	bool checkIgnoreFilters( const QString & entryName ) const;
+
+	/**
+	 * Read a cache file that was picked up along the way:
+	 *
+	 * If one of the non-directory entries of this directory was
+	 * ".qdirstat.cache.gz", open it, and if the toplevel entry in that
+	 * file matches the current path, read all the cache contents, kill all
+	 * pending read jobs for subdirectories of this directory and return
+	 * 'true'. In that case, the current read job is finished and deleted
+	 * (!), control needs to be returned to the caller, and using any data
+	 * members of this object is no longer safe (since they have just been
+	 * deleted).
+	 *
+	 * In all other cases, consider that entry as a plain file and return
+	 * 'false'.
+	 **/
+	bool readCacheFile( const QString & cacheFileName );
+
+	/**
+	 * Handle an error during lstat() of a directory entry.
+	 **/
+	void handleLstatError( const QString & entryName );
+
+	/**
+	 * Exclude the directory of this read job after it is almost completely
+	 * read. This is used when checking for exclude rules matching direct
+	 * file children of a directory.
+	 *
+	 * The main purpose of having this as a separate function is to have a
+	 * clear backtrace if it segfaults.
+	 **/
+	void excludeDirLate();
+
+	/**
+	 * Return the full name with path of an entry of this directory.
+	 **/
+	QString fullName( const QString & entryName ) const;
 
 
-	DIR * _diskDir;
+	//
+	// Data members
+	//
+
+	QString _dirName;
+	bool	_applyFileChildExcludeRules;
 
     };	// LocalDirReadJob
 
@@ -252,8 +329,8 @@ namespace QDirStat
 	 * Constructor for a cache reader that is already open.
 	 *
 	 * The CacheReadJob takes over ownership of the CacheReader. In
-	 * particular, the CacheReader will be destroyed with 'delete' when
-	 * the read job is done.
+	 * particular, the CacheReader will be destroyed with 'delete' when the
+	 * read job is done.
 	 *
 	 * If 'parent' is 0, the content of the cache file will replace all
 	 * current tree items.
@@ -280,7 +357,7 @@ namespace QDirStat
 	/**
 	 * Start reading the cache. Prior to this nothing happens.
 	 *
-	 * Inherited and reimplemented from @ref DirReadJob.
+	 * Inherited and reimplemented from DirReadJob.
 	 **/
 	virtual void read();
 
@@ -344,12 +421,26 @@ namespace QDirStat
 	/**
 	 * Count the number of pending jobs in the queue.
 	 **/
-	int count() const   { return _queue.count(); }
+	int count() const   { return _queue.count() + _blocked.count(); }
 
 	/**
 	 * Check if the queue is empty.
 	 **/
-	bool isEmpty() const { return _queue.isEmpty(); }
+	bool isEmpty() const { return _queue.isEmpty() && _blocked.isEmpty(); }
+
+	/**
+	 * Add a job to the list of blocked jobs: Jobs that are not yet ready
+	 * yet, e.g. because they are waiting for results from an external
+	 * process.
+	 **/
+	void addBlocked( DirReadJob * job );
+
+	/**
+	 * Notification that a job that was blocked is now ready to be
+	 * scheduled, so it will be taken out of the list of blocked jobs and
+	 * added to the end of the queue.
+	 **/
+	void unblock( DirReadJob * job );
 
 	/**
 	 * Clear the queue: Remove all pending jobs from the queue and destroy
@@ -374,6 +465,7 @@ namespace QDirStat
 	 **/
 	void jobFinishedNotify( DirReadJob *job );
 
+
     signals:
 
 	/**
@@ -391,13 +483,13 @@ namespace QDirStat
 
     public slots:
 
-        /**
-         * Notification that a child node is about to be deleted from the
-         * outside (i.e., not from this ReadJobQueue), e.g. because of cleanup
-         * actions. This will remove all pending directory read jobs for that
-         * subtree from the job queue.
-         **/
-        void deletingChildNotify( FileInfo * child );
+	/**
+	 * Notification that a child node is about to be deleted from the
+	 * outside (i.e., not from this ReadJobQueue), e.g. because of cleanup
+	 * actions. This will remove all pending directory read jobs for that
+	 * subtree from the job queue.
+	 **/
+	void deletingChildNotify( FileInfo * child );
 
 
     protected slots:
@@ -417,6 +509,7 @@ namespace QDirStat
     protected:
 
 	QList<DirReadJob *>  _queue;
+	QList<DirReadJob *>  _blocked;
 	QTimer		     _timer;
     };
 
@@ -426,14 +519,14 @@ namespace QDirStat
      **/
     inline QTextStream & operator<< ( QTextStream & str, DirReadJob * job )
     {
-        if ( job )
-        {
-            CacheReadJob * cacheReadJob = dynamic_cast<CacheReadJob *>( job );
-            QString jobType = cacheReadJob ? "CacheReadJob" : "DirReadJob";
-            str << "<" << jobType << " " << job->dir() << ">";
-        }
-        else
-            str << "<NULL DirReadJob *>";
+	if ( job )
+	{
+	    CacheReadJob * cacheReadJob = dynamic_cast<CacheReadJob *>( job );
+	    QString jobType = cacheReadJob ? "CacheReadJob" : "DirReadJob";
+	    str << "<" << jobType << " " << job->dir() << ">";
+	}
+	else
+	    str << "<NULL DirReadJob *>";
 
 	return str;
     }
