@@ -11,16 +11,25 @@
 #include <QFileInfo>
 
 #include "DirTree.h"
-#include "FileInfoSet.h"
-#include "Exception.h"
 #include "DirTreeCache.h"
+#include "DirTreeFilter.h"
+#include "DotEntry.h"
+#include "Attic.h"
+#include "FileInfoIterator.h"
+#include "FileInfoSet.h"
+#include "ExcludeRules.h"
+#include "PkgReader.h"
 #include "MountPoints.h"
+#include "Exception.h"
+
+#define VERBOSE_EXCLUDE_RULES	1
 
 using namespace QDirStat;
 
 
-DirTree::DirTree()
-    : QObject()
+DirTree::DirTree():
+    QObject(),
+    _excludeRules( 0 )
 {
     _isBusy	      = false;
     _crossFileSystems = false;
@@ -31,7 +40,7 @@ DirTree::DirTree()
 	     this,	  SLOT	( slotFinished() ) );
 
     connect( this,	  SIGNAL( deletingChild	     ( FileInfo * ) ),
-	     & _jobQueue, SLOT  ( deletingChildNotify( FileInfo * ) ) );
+	     & _jobQueue, SLOT	( deletingChildNotify( FileInfo * ) ) );
 }
 
 
@@ -39,6 +48,11 @@ DirTree::~DirTree()
 {
     if ( _root )
 	delete _root;
+
+    if ( _excludeRules )
+	delete _excludeRules;
+
+    clearFilters();
 }
 
 
@@ -52,6 +66,9 @@ void DirTree::setRoot( DirInfo *newRoot )
     }
 
     _root = newRoot;
+
+    FileInfo * realRoot = firstToplevel();
+    _url = realRoot ? realRoot->url() : "";
 }
 
 
@@ -69,9 +86,7 @@ bool DirTree::isTopLevel( FileInfo *item ) const
 
 QString DirTree::url() const
 {
-    FileInfo * realRoot = firstToplevel();
-
-    return realRoot ? realRoot->url() : "";
+    return _url;
 }
 
 
@@ -90,13 +105,21 @@ void DirTree::clear()
 }
 
 
+void DirTree::reset()
+{
+    clear();
+    clearExcludeRules();
+    clearFilters();
+}
+
+
 void DirTree::startReading( const QString & rawUrl )
 {
     QFileInfo fileInfo( rawUrl );
-    QString url = fileInfo.absoluteFilePath();
+    _url = fileInfo.absoluteFilePath();
     // logDebug() << "rawUrl: \"" << rawUrl << "\"" << endl;
-    logInfo() << "   url: \"" << url	 << "\"" << endl;
-    const MountPoint * mountPoint = MountPoints::findNearestMountPoint( url );
+    logInfo() << "   url: \"" << _url	 << "\"" << endl;
+    const MountPoint * mountPoint = MountPoints::findNearestMountPoint( _url );
     _device = mountPoint ? mountPoint->device() : "";
     logInfo() << "device: " << _device << endl;
 
@@ -106,7 +129,7 @@ void DirTree::startReading( const QString & rawUrl )
     _isBusy = true;
     emit startingReading();
 
-    FileInfo * item = LocalDirReadJob::stat( url, this, _root );
+    FileInfo * item = LocalDirReadJob::stat( _url, this, _root );
     CHECK_PTR( item );
 
     if ( item )
@@ -120,6 +143,7 @@ void DirTree::startReading( const QString & rawUrl )
 	}
 	else
 	{
+	    finalizeTree();
 	    _isBusy = false;
 	    emit readJobFinished( _root );
 	    emit finished();
@@ -127,10 +151,9 @@ void DirTree::startReading( const QString & rawUrl )
     }
     else	// stat() failed
     {
-	logWarning() << "stat(" << url << ") failed" << endl;
+	logWarning() << "stat(" << _url << ") failed" << endl;
 	_isBusy = false;
 	emit finished();
-	emit finalizeLocal( 0 );
     }
 }
 
@@ -189,12 +212,7 @@ void DirTree::refresh( DirInfo * subtree )
     {
 	// logDebug() << "Refreshing subtree " << subtree << endl;
 
-	if ( subtree->hasChildren() )
-	{
-	    emit clearingSubtree( subtree );
-	    subtree->clear();
-	    emit subtreeCleared( subtree );
-	}
+	clearSubtree( subtree );
 
 	subtree->reset();
 	subtree->setExcluded( false );
@@ -219,8 +237,22 @@ void DirTree::abortReading()
 }
 
 
+void DirTree::finalizeTree()
+{
+    if ( _root && hasFilters() )
+    {
+	recalc( _root );
+	ignoreEmptyDirs( _root );
+	recalc( _root );
+	moveIgnoredToAttic( _root );
+	recalc( _root );
+    }
+}
+
+
 void DirTree::slotFinished()
 {
+    finalizeTree();
     _isBusy = false;
     emit finished();
 }
@@ -273,7 +305,7 @@ void DirTree::deleteSubtree( FileInfo *subtree )
 		    // logDebug() << "Removing empty dot entry " << parent << endl;
 
 		    deletingChildNotify( parent );
-		    parent->parent()->setDotEntry( 0 );
+		    parent->parent()->deleteEmptyDotEntry();
 
 		    delete parent;
 		    parent = 0;
@@ -312,15 +344,32 @@ void DirTree::deleteSubtree( FileInfo *subtree )
 }
 
 
+void DirTree::clearSubtree( DirInfo * subtree )
+{
+    if ( subtree->hasChildren() )
+    {
+	emit clearingSubtree( subtree );
+	subtree->clear();
+	emit subtreeCleared( subtree );
+    }
+}
+
+
 void DirTree::addJob( DirReadJob * job )
 {
     _jobQueue.enqueue( job );
 }
 
 
-void DirTree::sendFinalizeLocal( DirInfo *dir )
+void DirTree::addBlockedJob( DirReadJob * job )
 {
-    emit finalizeLocal( dir );
+    _jobQueue.addBlocked( job );
+}
+
+
+void DirTree::unblock( DirReadJob * job )
+{
+    _jobQueue.unblock( job );
 }
 
 
@@ -332,12 +381,15 @@ void DirTree::sendStartingReading()
 
 void DirTree::sendFinished()
 {
+    finalizeTree();
+    _isBusy = false;
     emit finished();
 }
 
 
 void DirTree::sendAborted()
 {
+    _isBusy = false;
     emit aborted();
 }
 
@@ -355,6 +407,24 @@ void DirTree::sendReadJobFinished( DirInfo * dir )
 }
 
 
+FileInfo * DirTree::locate( QString url, bool findPseudoDirs )
+{
+    if ( ! _root )
+	return 0;
+
+    FileInfo * topItem = _root->firstChild();
+
+    if ( topItem	      &&
+	 topItem->isPkgInfo() &&
+	 topItem->url() == url	)
+    {
+	return topItem;
+    }
+
+    return _root->locate( url, findPseudoDirs );
+}
+
+
 bool DirTree::writeCache( const QString & cacheFileName )
 {
     CacheWriter writer( cacheFileName.toUtf8(), this );
@@ -370,5 +440,203 @@ void DirTree::readCache( const QString & cacheFileName )
 }
 
 
+void DirTree::clearAndReadCache( const QString & cacheFileName )
+{
+    clear();
+    readCache( cacheFileName );
+}
+
+
+void DirTree::readPkg( const PkgFilter & pkgFilter )
+{
+    clear();
+    _isBusy = true;
+    _url    = pkgFilter.url();
+    emit startingReading();
+
+    // logDebug() << "Reading " << pkgFilter << endl;
+    PkgReader reader( this );
+    reader.read( pkgFilter );
+}
+
+
+void DirTree::setExcludeRules( ExcludeRules * newRules )
+{
+    if ( _excludeRules )
+	delete _excludeRules;
+
+#if VERBOSE_EXCLUDE_RULES
+    if ( newRules )
+    {
+	logDebug() << "New tmp exclude rules:" << endl;
+
+	for ( ExcludeRuleListIterator it = newRules->begin(); it != newRules->end(); ++it )
+	{
+	    logDebug() << *it << endl;
+	}
+    }
+    else
+    {
+	logDebug() << "Clearing tmp exclude rules" << endl;
+    }
+#endif
+
+    _excludeRules = newRules;
+}
+
+
+void DirTree::addFilter( DirTreeFilter * filter )
+{
+    if ( filter )
+	_filters << filter;
+}
+
+
+void DirTree::clearFilters()
+{
+    qDeleteAll( _filters );
+    _filters.clear();
+}
+
+
+bool DirTree::checkIgnoreFilters( const QString & path )
+{
+    foreach ( DirTreeFilter * filter, _filters )
+    {
+	if ( filter->ignore( path ) )
+	    return true;
+    }
+
+    return false;
+}
+
+
+void DirTree::moveIgnoredToAttic( DirInfo * dir )
+{
+    CHECK_PTR( dir );
+
+    if ( dir->totalIgnoredItems() == 0 && dir->totalUnignoredItems() > 0 )
+	return;
+
+    // Not using FileInfoIterator because we don't want to iterate over the dot
+    // entry as well, just the normal children.
+
+    FileInfo * child = dir->firstChild();
+    FileInfoList ignoredChildren;
+
+    while ( child )
+    {
+	if ( child->isIgnored() )
+	{
+	    // Don't move the child right here, otherwise the iteration breaks
+	    ignoredChildren << child;
+	}
+	else
+	{
+	    if ( child->isDirInfo() )
+		moveIgnoredToAttic( child->toDirInfo() );
+	}
+
+	child = child->next();
+    }
+
+
+    foreach ( FileInfo * child, ignoredChildren )
+    {
+	// logDebug() << "Moving ignored " << child << " to attic" << endl;
+	dir->moveToAttic( child );
+
+	if ( child->isDirInfo() )
+	    unatticAll( child->toDirInfo() );
+    }
+
+    if ( ! ignoredChildren.isEmpty() )
+    {
+	dir->recalc();
+
+	if ( dir->attic() )
+	    dir->attic()->recalc();
+    }
+}
+
+
+void DirTree::ignoreEmptyDirs( DirInfo * dir )
+{
+    CHECK_PTR( dir );
+
+    FileInfo * child = dir->firstChild();
+    FileInfoList ignoredChildren;
+
+    while ( child )
+    {
+	if ( ! child->isIgnored() && child->isDirInfo() )
+	{
+	    DirInfo * subDir = child->toDirInfo();
+
+	    if ( subDir->totalUnignoredItems() == 0 )
+		// && ! subDir->isMountPoint()
+	    {
+		// logDebug() << "Ignoring empty subdir " << subDir << endl;
+		subDir->setIgnored( true );
+	    }
+	    else
+	    {
+		ignoreEmptyDirs( subDir );
+	    }
+	}
+
+	child = child->next();
+    }
+}
+
+
+void DirTree::unatticAll( DirInfo * dir )
+{
+    CHECK_PTR( dir );
+
+    if ( dir->attic() )
+    {
+	// logDebug() << "Moving all attic children to the normal children list for " << dir << endl;
+	dir->takeAllChildren( dir->attic() );
+	dir->deleteEmptyAttic();
+	dir->recalc();
+    }
+
+    FileInfoIterator it( dir );
+
+    while ( *it )
+    {
+	if ( (*it)->isDirInfo() )
+	    unatticAll( (*it)->toDirInfo() );
+
+	++it;
+    }
+}
+
+
+void DirTree::recalc( DirInfo * dir )
+{
+    CHECK_PTR( dir );
+
+    FileInfo * child = dir->firstChild();
+
+    while ( child )
+    {
+	if ( child->isDirInfo() )
+	    recalc( child->toDirInfo() );
+
+	child = child->next();
+    }
+
+    if ( dir->dotEntry() )
+	recalc( dir->dotEntry() );
+
+    if ( dir->attic() )
+	recalc( dir->attic() );
+
+    dir->recalc();
+}
+
+
 
 // EOF
