@@ -17,6 +17,7 @@
 #include "DataColumns.h"
 #include "SelectionModel.h"
 #include "Settings.h"
+#include "SettingsHelpers.h"
 #include "Logger.h"
 #include "Exception.h"
 #include "DebugHelpers.h"
@@ -66,6 +67,25 @@ void DirTreeModel::readSettings()
     _slowUpdateMillisec	 = settings.value( "SlowUpdateMillisec", 3000 ).toInt();
 
     settings.endGroup();
+
+    if ( usingLightTheme() )
+    {
+	settings.beginGroup( "TreeTheme-light" );
+
+	_dirReadErrColor     = readColorEntry( settings, "DirReadErrColor",	QColor( Qt::red		 ) );
+	_subtreeReadErrColor = readColorEntry( settings, "SubtreeReadErrColor", QColor( 0xa0, 0x00, 0x00 ) );
+
+	settings.endGroup();
+    }
+    else // dark theme
+    {
+	settings.beginGroup( "TreeTheme-dark" );
+
+	_dirReadErrColor     = readColorEntry( settings, "DirReadErrColor",	QColor( Qt::red		 ) );
+	_subtreeReadErrColor = readColorEntry( settings, "SubtreeReadErrColor", QColor( Qt::yellow	 ) );
+
+	settings.endGroup();
+    }
 }
 
 
@@ -77,10 +97,26 @@ void DirTreeModel::writeSettings()
     settings.setValue( "SlowUpdateMillisec", _slowUpdateMillisec  );
 
     settings.setDefaultValue( "CrossFileSystems",    _tree ? _tree->crossFileSystems() : false );
-    settings.setDefaultValue( "TreeIconDir" ,	     _treeIconDir	  );
+    settings.setDefaultValue( "TreeIconDir",	     _treeIconDir	  );
     settings.setDefaultValue( "UpdateTimerMillisec", _updateTimerMillisec );
 
     settings.endGroup();
+
+
+    settings.beginGroup( usingLightTheme() ? "TreeTheme-light" : "TreeTheme-dark" );
+
+    writeColorEntry( settings, "DirReadErrColor",     _dirReadErrColor	   );
+    writeColorEntry( settings, "SubtreeReadErrColor", _subtreeReadErrColor );
+
+    settings.endGroup();
+}
+
+
+bool DirTreeModel::usingDarkTheme()
+{
+    QColor background = qAppPalette().color( QPalette::Active, QPalette::Base );
+
+    return background.lightness() < 128; // 0 (black) .. 255 (white)
 }
 
 
@@ -191,9 +227,6 @@ void DirTreeModel::loadIcons()
     _charDeviceIcon    = QPixmap( _treeIconDir + "char-device.png"    );
     _specialIcon       = QPixmap( _treeIconDir + "special.png"	      );
     _pkgIcon	       = QPixmap( _treeIconDir + "folder-pkg.png"     );
-#if 0
-    _pkgIcon	       = QPixmap( _treeIconDir + "package.png"	      );
-#endif
 }
 
 
@@ -310,6 +343,7 @@ int DirTreeModel::rowCount( const QModelIndex & parentIndex ) const
 	    break;
 
 	case DirError:
+        case DirPermissionDenied:
 
 	    // This is a hybrid case: Depending on the dir reader, the dir may
 	    // or may not be finished at this time. For a local dir, it most
@@ -369,13 +403,22 @@ QVariant DirTreeModel::data( const QModelIndex & index, int role ) const
 		return result;
 	    }
 
-        case Qt::ForegroundRole:
-            {
-                if ( item->isIgnored() || item->isAttic() )
-                    return qAppPalette().brush( QPalette::Disabled, QPalette::Foreground );
-                else
-                    return QVariant();
-            }
+	case Qt::ForegroundRole:
+	    {
+		if ( item->isIgnored() || item->isAttic() )
+		    return qAppPalette().brush( QPalette::Disabled, QPalette::Foreground );
+
+		if ( item->isDir() )
+		{
+		    if ( item->readError() )
+			return _dirReadErrColor;
+
+		    if ( item->errSubDirCount() > 0 )
+			return _subtreeReadErrColor;
+		}
+
+		return QVariant();
+	    }
 
 	case Qt::DecorationRole:
 	    {
@@ -401,6 +444,7 @@ QVariant DirTreeModel::data( const QModelIndex & index, int role ) const
 
 		    case NameCol:
 		    case LatestMTimeCol:
+		    case OldestFileMTimeCol:
 		    case UserCol:
 		    case GroupCol:
 		    default:
@@ -424,7 +468,7 @@ QVariant DirTreeModel::data( const QModelIndex & index, int role ) const
 			{
 			    if ( ( item->parent() && item->parent()->isBusy() ) ||
 				 item == _tree->firstToplevel() ||
-                                 item->isAttic() )
+				 item->isAttic() )
 			    {
 				return -1.0;
 			    }
@@ -439,6 +483,7 @@ QVariant DirTreeModel::data( const QModelIndex & index, int role ) const
 		    case TotalFilesCol:	      return item->totalFiles();
 		    case TotalSubDirsCol:     return item->totalSubDirs();
 		    case LatestMTimeCol:      return (qulonglong) item->latestMtime();
+		    case OldestFileMTimeCol:  return (qulonglong) item->oldestFileMtime();
 		    case UserCol:	      return item->uid();
 		    case GroupCol:	      return item->gid();
 		    case PermissionsCol:      return item->mode();
@@ -476,6 +521,7 @@ QVariant DirTreeModel::headerData( int		   section,
 		case TotalFilesCol:	  return tr( "Files"		  );
 		case TotalSubDirsCol:	  return tr( "Subdirs"		  );
 		case LatestMTimeCol:	  return tr( "Last Modified"	  );
+		case OldestFileMTimeCol:  return tr( "Oldest File"	  );
 		case UserCol:		  return tr( "User"		  );
 		case GroupCol:		  return tr( "Group"		  );
 		case PermissionsCol:	  return tr( "Permissions"	  );
@@ -493,6 +539,7 @@ QVariant DirTreeModel::headerData( int		   section,
 		case TotalFilesCol:
 		case TotalSubDirsCol:
 		case LatestMTimeCol:
+		case OldestFileMTimeCol:
 		case PermissionsCol:
 		case OctalPermissionsCol: return Qt::AlignHCenter;
 		default:		  return Qt::AlignLeft;
@@ -659,7 +706,15 @@ QVariant DirTreeModel::columnText( FileInfo * item, int col ) const
     bool limitedInfo = item->isPseudoDir() || item->readState() == DirCached || item->isPkgInfo();
 
     if ( item->isAttic() && col == PercentNumCol )
-        return QVariant();
+	return QVariant();
+
+    if ( item->isPkgInfo() &&
+         item->readState() == DirAborted &&
+         ! item->firstChild() &&
+         col != NameCol )
+    {
+        return "?";
+    }
 
     switch ( col )
     {
@@ -676,7 +731,21 @@ QVariant DirTreeModel::columnText( FileInfo * item, int col ) const
 
     if ( item->isDirInfo() )
     {
-	QString prefix = item->readState() == DirAborted ? ">" : "";
+	if ( item->readError() )
+	{
+            switch ( col )
+            {
+                case TotalItemsCol:
+                case TotalFilesCol:
+                case TotalSubDirsCol:
+                    return "?";
+
+                default:
+                    break;
+            }
+	}
+
+	QString prefix = item->sizePrefix();
 
 	switch ( col )
 	{
@@ -687,6 +756,8 @@ QVariant DirTreeModel::columnText( FileInfo * item, int col ) const
 		    return QVariant();
 		else
 		    return prefix + QString( "%1" ).arg( item->totalSubDirs() );
+
+	    case OldestFileMTimeCol:  return QString( "  " ) + formatTime( item->oldestFileMtime() );
 	}
     }
 
@@ -708,48 +779,55 @@ int DirTreeModel::directChildrenCount( FileInfo * subtree ) const
 }
 
 
+QString DirTreeModel::sizeText( FileInfo * item, QString (*fmtSz)(FileSize) )
+{
+    if ( ! item->isFile() )
+        return "";
+
+    QString text;
+
+    if ( item->links() > 1 )  // Multiple hard links
+    {
+        if ( item->isSparseFile() )
+        {
+            text = tr( "%1 / %2 Links (allocated: %3)" )
+                .arg( fmtSz( item->byteSize() ) )
+                .arg( item->links() )
+                .arg( fmtSz( item->allocatedSize() ) );
+        }
+        else
+        {
+            text = tr( "%1 / %2 Links" )
+                .arg( fmtSz( item->byteSize() ) )
+                .arg( item->links() );
+        }
+    }
+    else // No multiple hard links
+    {
+        if ( item->isSparseFile() )
+        {
+            text = tr( "%1 (allocated: %2)" )
+                .arg( fmtSz( item->byteSize() ) )
+                .arg( fmtSz( item->allocatedSize() ) );
+        }
+    }
+
+    return text;
+}
+
+
 QVariant DirTreeModel::sizeColText( FileInfo * item ) const
 {
     if ( item->isDevice() )
 	return QVariant();
 
     if ( item->isDirInfo() )
-    {
-	QString prefix = item->readState() == DirAborted ? ">" : "";
-	return prefix + formatSize( item->totalSize() );
-    }
+	return item->sizePrefix() + formatSize( item->totalSize() );
 
-    QString text;
+    QString text = sizeText( item );
 
-    if ( item->isFile() && ( item->links() > 1 ) ) // Regular file with multiple links
-    {
-	if ( item->isSparseFile() )
-	{
-	    text = tr( "%1 / %2 Links (allocated: %3)" )
-		.arg( formatSize( item->byteSize() ) )
-		.arg( formatSize( item->links() ) )
-		.arg( formatSize( item->allocatedSize() ) );
-	}
-	else
-	{
-	    text = tr( "%1 / %2 Links" )
-		.arg( formatSize( item->byteSize() ) )
-		.arg( item->links() );
-	}
-    }
-    else // No multiple links or no regular file
-    {
-	if ( item->isSparseFile() )
-	{
-	    text = tr( "%1 (allocated: %2)" )
-		.arg( formatSize( item->byteSize() ) )
-		.arg( formatSize( item->allocatedSize() ) );
-	}
-	else
-	{
-	    text = formatSize( item->size() );
-	}
-    }
+    if ( text.isEmpty() )
+        text = formatSize( item->size() );
 
     return text;
 }
@@ -768,13 +846,13 @@ QVariant DirTreeModel::columnIcon( FileInfo * item, int col ) const
     else if ( item->isDir()	 )
     {
 	if	( item->readState() == DirAborted )   icon = _stopIcon;
-	else if ( item->readState() == DirError	  )   icon = _unreadableDirIcon;
+	else if ( item->readError()       	  )   icon = _unreadableDirIcon;
 	else if ( item->isMountPoint()		  )   icon = _mountPointIcon;
 	else					      icon = _dirIcon;
     }
     else // ! item->isDir()
     {
-	if	( item->readState() == DirError	  )   icon = _unreadableDirIcon;
+	if	( item->readError()     	  )   icon = _unreadableDirIcon;
 	else if ( item->isFile()		  )   icon = _fileIcon;
 	else if ( item->isSymLink()		  )   icon = _symlinkIcon;
 	else if ( item->isBlockDevice()		  )   icon = _blockDeviceIcon;
@@ -1094,7 +1172,8 @@ QVariant DirTreeModel::formatPercent( float percent ) const
 	return QVariant();
 
     QString text;
-    text.sprintf( "%.1f%%", percent );
+    text.setNum( percent, 'f', 1 );
+    text += "%";
 
     return text;
 }
