@@ -72,17 +72,28 @@ void DirTreeView::contextMenu( const QPoint & pos )
 	return;
     }
 
+    DataColumn col  = DataColumns::fromViewCol( index.column() );
+
+    if ( col == SizeCol )
+	sizeContextMenu( pos, index );
+    else
+	actionContextMenu( pos );
+}
+
+
+void DirTreeView::actionContextMenu( const QPoint & pos )
+{
     QMenu menu;
     QStringList actions;
     actions << "actionGoUp"
-	    << "actionCopyUrlToClipboard"
+	    << "actionCopyPathToClipboard"
 	    << "---"
 	    << "actionRefreshSelected"
 	    << "actionReadExcludedDirectory"
 	    << "actionContinueReadingAtMountPoint"
 	    << "---"
-            << "actionFileSizeStats"
-            << "actionFileTypeStats"
+	    << "actionFileSizeStats"
+	    << "actionFileTypeStats"
 	    << "---"
 	    << "actionMoveToTrash"
 	;
@@ -99,6 +110,25 @@ void DirTreeView::contextMenu( const QPoint & pos )
 }
 
 
+void DirTreeView::sizeContextMenu( const QPoint & pos, const QModelIndex & index )
+{
+    FileInfo * item = static_cast<FileInfo *>( index.internalPointer() );
+    CHECK_MAGIC( item );
+
+    if ( item->byteSize() >= 1024 )  // Not useful below 1 kB
+    {
+        QString text = DirTreeModel::sizeText( item, formatByteSize );
+
+        if ( text.isEmpty() )
+            text = item->sizePrefix() + formatByteSize( item->totalSize() );
+
+        QMenu menu;
+        menu.addAction( text );
+        menu.exec( mapToGlobal( pos ) );
+    }
+}
+
+
 QModelIndexList DirTreeView::expandedIndexes() const
 {
     QModelIndexList expandedList;
