@@ -9,10 +9,14 @@
 #include "FileDetailsView.h"
 #include "AdaptiveTimer.h"
 #include "DirInfo.h"
+#include "DirTreeModel.h"
 #include "FileInfoSet.h"
 #include "MimeCategorizer.h"
 #include "PkgQuery.h"
 #include "SystemFileChecker.h"
+#include "Settings.h"
+#include "SettingsHelpers.h"
+#include "Qt4Compat.h"
 #include "Logger.h"
 #include "Exception.h"
 
@@ -30,7 +34,9 @@ FileDetailsView::FileDetailsView( QWidget * parent ):
 
     _ui->setupUi( this );
     clear();
+    readSettings();
 
+    _normalTextColor = qAppPalette().color( QPalette::Active, QPalette::WindowText );
     _labelLimit = 0; // Unlimited
     // TO DO: Read _labelLimit from the config file
 
@@ -39,18 +45,18 @@ FileDetailsView::FileDetailsView( QWidget * parent ):
     _pkgUpdateTimer->addDelayStage(  800 ); // millisec
     _pkgUpdateTimer->addDelayStage( 1200 ); // millisec
 
-    _pkgUpdateTimer->addCoolDownPeriod(  500 ); // millisec
+    _pkgUpdateTimer->addCoolDownPeriod(	 500 ); // millisec
     _pkgUpdateTimer->addCoolDownPeriod( 1500 ); // millisec
     _pkgUpdateTimer->addCoolDownPeriod( 3000 ); // millisec
 
     connect( _pkgUpdateTimer, SIGNAL( deliverRequest( QVariant ) ),
-             this,            SLOT  ( updatePkgInfo ( QVariant ) ) );
+	     this,	      SLOT  ( updatePkgInfo ( QVariant ) ) );
 }
 
 
 FileDetailsView::~FileDetailsView()
 {
-
+    writeSettings();
 }
 
 
@@ -68,7 +74,7 @@ void FileDetailsView::setCurrentPage( QWidget *page )
     // the QStackedWidget, but no longer in the layout.
 
     while ( count() > 0 )
-        removeWidget( widget( 0 ) );
+	removeWidget( widget( 0 ) );
 
     addWidget( page );
     setCurrentWidget( page );
@@ -109,7 +115,7 @@ void FileDetailsView::showDetails( FileInfo * file )
 
     if ( file->isPkgInfo() )
     {
-        showDetails( file->toPkgInfo() );
+	showDetails( file->toPkgInfo() );
     }
     else if ( file->isDirInfo() )
     {
@@ -122,27 +128,27 @@ void FileDetailsView::showDetails( FileInfo * file )
 	setCurrentPage( _ui->fileDetailsPage );
 	showFileInfo( file );
 
-        PkgInfo * pkg = file->pkgInfoParent();
+	PkgInfo * pkg = file->pkgInfoParent();
 
-        if ( pkg )
-        {
-            // The simple case: We are in package view mode, and this file has
-            // a PkgInfo ancestor. So we know that this belongs to a package,
-            // also to which one, and that this is inherently a system file
-            // since it belongs to a package.
+	if ( pkg )
+	{
+	    // The simple case: We are in package view mode, and this file has
+	    // a PkgInfo ancestor. So we know that this belongs to a package,
+	    // also to which one, and that this is inherently a system file
+	    // since it belongs to a package.
 
-            setSystemFileWarningVisibility( true );
-            _ui->filePackageLabel->setEnabled( true );
-            _ui->filePackageLabel->setText( pkg->name() );
-        }
-        else
-        {
-            // The not-so-simple case: Use heuristics to check if this file is
-            // in a system directory and then query the package manager after a
-            // timeout what (if any) package this file belongs to.
+	    setSystemFileWarningVisibility( true );
+	    _ui->filePackageLabel->setEnabled( true );
+	    _ui->filePackageLabel->setText( pkg->name() );
+	}
+	else
+	{
+	    // The not-so-simple case: Use heuristics to check if this file is
+	    // in a system directory and then query the package manager after a
+	    // timeout what (if any) package this file belongs to.
 
-            showFilePkgInfo( file );
-        }
+	    showFilePkgInfo( file );
+	}
     }
 }
 
@@ -160,7 +166,7 @@ void FileDetailsView::showFileInfo( FileInfo * file )
     _ui->fileMimeCategoryLabel->setEnabled( ! category.isEmpty() );
     _ui->fileMimeCategoryLabel->setText( category );
 
-    _ui->fileSizeLabel->setText( fileSizeText( file ) );
+    setFileSizeLabel( _ui->fileSizeLabel, file );
     _ui->fileUserLabel->setText( file->userName() );
     _ui->fileGroupLabel->setText( file->groupName() );
     _ui->filePermissionsLabel->setText( formatPermissions( file->mode() ) );
@@ -169,45 +175,24 @@ void FileDetailsView::showFileInfo( FileInfo * file )
 }
 
 
-QString FileDetailsView::fileSizeText( FileInfo * file ) const
+void FileDetailsView::setFileSizeLabel( FileSizeLabel * label,
+					FileInfo *	file )
 {
     CHECK_PTR( file );
 
-    // Taken from DirTreeModel::sizeColText()
+    QString text = DirTreeModel::sizeText( file );
 
-    QString text;
-
-    if ( file->isFile() && ( file->links() > 1 ) ) // Regular file with multiple links
+    if ( text.isEmpty() )  // The normal case: No hard links, not a sparse file
     {
-	if ( file->isSparseFile() )
-	{
-	    text = tr( "%1 / %2 Links (allocated: %3)" )
-		.arg( formatSize( file->byteSize() ) )
-		.arg( formatSize( file->links() ) )
-		.arg( formatSize( file->allocatedSize() ) );
-	}
-	else
-	{
-	    text = tr( "%1 / %2 Links" )
-		.arg( formatSize( file->byteSize() ) )
-		.arg( file->links() );
-	}
+	label->setValue( file->size() );
     }
-    else // No multiple links or no regular file
+    else  // The exotic case: Multiple hard links or sparse file or both
     {
-	if ( file->isSparseFile() )
-	{
-	    text = tr( "%1 (allocated: %2)" )
-		.arg( formatSize( file->byteSize() ) )
-		.arg( formatSize( file->allocatedSize() ) );
-	}
-	else
-	{
-	    text = formatSize( file->size() );
-	}
-    }
+	label->setText( text );
 
-    return text;
+	if ( file->byteSize() >= 1024 ) // Not useful below 1 kB
+	    label->setContextText( DirTreeModel::sizeText( file, formatByteSize ) );
+    }
 }
 
 
@@ -220,20 +205,20 @@ void FileDetailsView::showFilePkgInfo( FileInfo * file )
 
     if ( PkgQuery::foundSupportedPkgManager() )
     {
-        setFilePkgBlockVisibility( isSystemFile );
+	setFilePkgBlockVisibility( isSystemFile );
 
-            if ( isSystemFile )
-            {
-                QString delayHint = QString( _pkgUpdateTimer->delayStage(), '.' );
-                _ui->filePackageLabel->setText( delayHint );
+	    if ( isSystemFile )
+	    {
+		QString delayHint = QString( _pkgUpdateTimer->delayStage(), '.' );
+		_ui->filePackageLabel->setText( delayHint );
 
-                _ui->filePackageCaption->setEnabled( true );
-                _pkgUpdateTimer->delayedRequest( file->url() );
-            }
+		_ui->filePackageCaption->setEnabled( true );
+		_pkgUpdateTimer->delayedRequest( file->url() );
+	    }
     }
     else // No supported package manager found
     {
-        setFilePkgBlockVisibility( false );
+	setFilePkgBlockVisibility( false );
     }
 }
 
@@ -280,6 +265,9 @@ void FileDetailsView::showDetails( DirInfo * dir )
     setLabelLimited(_ui->dirNameLabel, name );
     _ui->dirTypeLabel->setText( dirType );
 
+    _ui->dirIcon->setVisible( ! dir->readError() );
+    _ui->dirUnreadableIcon->setVisible( dir->readError() );
+
     // Subtree information
 
     showSubtreeInfo( dir );
@@ -290,44 +278,51 @@ void FileDetailsView::showDetails( DirInfo * dir )
 void FileDetailsView::showSubtreeInfo( DirInfo * dir )
 {
     CHECK_PTR( dir );
+    QString msg;
 
-    if ( ! dir->isBusy() &&
-         ( dir->readState() == DirFinished ||
-           dir->readState() == DirCached     ) )
+    _ui->dirLockedIcon->setVisible( dir->readError() );
+
+    switch ( dir->readState() )
     {
-        setLabel( _ui->dirTotalSizeLabel,   dir->totalSize()    );
-        setLabel( _ui->dirItemCountLabel,   dir->totalItems()   );
-        setLabel( _ui->dirFileCountLabel,   dir->totalFiles()   );
-        setLabel( _ui->dirSubDirCountLabel, dir->totalSubDirs() );
-    }
-    else
-    {
-        QString msg;
+	case DirQueued:
+	case DirReading:		msg = tr( "[Reading]"		); break;
 
-        if ( dir->isBusy() )
-        {
-            msg = tr( "[Reading]" );
-        }
-        else
-        {
-            switch ( dir->readState() )
-            {
-                case DirOnRequestOnly: msg = tr( "[Not Read]"   ); break;
-                case DirAborted:       msg = tr( "[Aborted]"    ); break;
-                case DirError:         msg = tr( "[Read Error]" ); break;
+	case DirOnRequestOnly:		msg = tr( "[Not Read]"		); break;
+	case DirPermissionDenied:	msg = tr( "[Permission Denied]" ); break;
+	case DirError:			msg = tr( "[Read Error]"	); break;
 
-                default:
-                    break;
-            }
-        }
-
-        _ui->dirTotalSizeLabel->setText( msg );
-        _ui->dirItemCountLabel->clear();
-        _ui->dirFileCountLabel->clear();
-        _ui->dirSubDirCountLabel->clear();
+	case DirFinished:
+	case DirCached:
+	case DirAborted:
+	    break;
     }
 
-    _ui->dirLatestMTimeLabel->setText( formatTime( dir->latestMtime() ) );
+#if 0
+    // Also displaying the error message in read seems a bit over the top.
+    setLabelColor( _ui->dirTotalSizeLabel,
+                   dir->readError() ? _dirReadErrColor : _normalTextColor );
+#endif
+
+    if ( msg.isEmpty() )
+    {
+	// No special msg -> show summary fields
+
+	QString prefix = dir->sizePrefix();
+
+	setLabel( _ui->dirTotalSizeLabel,   dir->totalSize(),	 prefix );
+	setLabel( _ui->dirItemCountLabel,   dir->totalItems(),	 prefix );
+	setLabel( _ui->dirFileCountLabel,   dir->totalFiles(),	 prefix );
+	setLabel( _ui->dirSubDirCountLabel, dir->totalSubDirs(), prefix );
+        _ui->dirLatestMTimeLabel->setText( formatTime( dir->latestMtime() ) );
+    }
+    else  // Special msg -> show it and clear all summary fields
+    {
+	_ui->dirTotalSizeLabel->setText( msg );
+	_ui->dirItemCountLabel->clear();
+	_ui->dirFileCountLabel->clear();
+	_ui->dirSubDirCountLabel->clear();
+        _ui->dirLatestMTimeLabel->clear();
+    }
 }
 
 
@@ -338,18 +333,25 @@ void FileDetailsView::showDirNodeInfo( DirInfo * dir )
 
     if ( ! dir->isPseudoDir() )
     {
-        _ui->dirOwnSizeCaption->setVisible( dir->size() > 0 );
-        _ui->dirOwnSizeLabel->setVisible  ( dir->size() > 0 );
+	_ui->dirOwnSizeCaption->setVisible( dir->size() > 0 );
+	_ui->dirOwnSizeLabel->setVisible  ( dir->size() > 0 );
 	setLabel( _ui->dirOwnSizeLabel, dir->size() );
 
 	_ui->dirUserLabel->setText( dir->userName() );
 	_ui->dirGroupLabel->setText( dir->groupName() );
 	_ui->dirPermissionsLabel->setText( formatPermissions( dir->mode() ) );
 
-
-        _ui->dirMTimeCaption->setVisible( dir->mtime() > 0 );
-        _ui->dirMTimeLabel->setVisible  ( dir->mtime() > 0);
+	_ui->dirMTimeCaption->setVisible( dir->mtime() > 0 );
+	_ui->dirMTimeLabel->setVisible	( dir->mtime() > 0);
 	_ui->dirMTimeLabel->setText( formatTime( dir->mtime() ) );
+
+
+        // Show permissions in read if there was a "permission denied" error
+        // while reading this directory
+
+        setLabelColor( _ui->dirPermissionsLabel,
+                       dir->readState() == DirPermissionDenied ?
+                       _dirReadErrColor : _normalTextColor );
     }
 }
 
@@ -388,8 +390,8 @@ void FileDetailsView::showDetails( PkgInfo * pkg )
 
     if ( pkg->url() == "Pkg:/" )
     {
-        showPkgSummary( pkg );
-        return;
+	showPkgSummary( pkg );
+	return;
     }
 
     setCurrentPage( _ui->pkgDetailsPage );
@@ -398,31 +400,43 @@ void FileDetailsView::showDetails( PkgInfo * pkg )
     _ui->pkgVersionLabel->setText( pkg->version() );
     _ui->pkgArchLabel->setText( pkg->arch() );
 
-    if ( ! pkg->isBusy() && pkg->readState() == DirFinished )
+    QString msg;
+
+    switch ( pkg->readState() )
     {
-        setLabel( _ui->pkgTotalSizeLabel,   pkg->totalSize()    );
-        setLabel( _ui->pkgItemCountLabel,   pkg->totalItems()   );
-        setLabel( _ui->pkgFileCountLabel,   pkg->totalFiles()   );
-        setLabel( _ui->pkgSubDirCountLabel, pkg->totalSubDirs() );
+	case DirQueued:
+	case DirReading:	   msg = tr( "[Reading]"	   ); break;
+
+	case DirPermissionDenied:  msg = tr( "[Permission Denied]" ); break;
+	case DirError:		   msg = tr( "[Read Error]"	   ); break;
+	case DirAborted:	   msg = tr( "[Aborted]"	   ); break;
+
+	case DirFinished:
+	    break;
+
+	case DirCached:
+	case DirOnRequestOnly:
+	    logError() << "Invalid readState for a Pkg" << endl;
+	    break;
     }
-    else
+
+    if ( msg.isEmpty() )
     {
-        QString msg;
+	// No special msg -> show summary fields
 
-        if ( pkg->isBusy() )
-        {
-            msg = tr( "[Reading]" );
-        }
-        else
-        {
-            if ( pkg->readState() == DirError )
-                msg = tr( "[Read Error]" );
-        }
+	QString prefix = pkg->sizePrefix();
 
-        _ui->pkgTotalSizeLabel->setText( msg );
-        _ui->pkgItemCountLabel->clear();
-        _ui->pkgFileCountLabel->clear();
-        _ui->pkgSubDirCountLabel->clear();
+	setLabel( _ui->pkgTotalSizeLabel,   pkg->totalSize()	);
+	setLabel( _ui->pkgItemCountLabel,   pkg->totalItems()	);
+	setLabel( _ui->pkgFileCountLabel,   pkg->totalFiles()	);
+	setLabel( _ui->pkgSubDirCountLabel, pkg->totalSubDirs() );
+    }
+    else  // Special msg -> show it and clear all summary fields
+    {
+	_ui->pkgTotalSizeLabel->setText( msg );
+	_ui->pkgItemCountLabel->clear();
+	_ui->pkgFileCountLabel->clear();
+	_ui->pkgSubDirCountLabel->clear();
     }
 
     _ui->pkgLatestMTimeLabel->setText( formatTime( pkg->latestMtime() ) );
@@ -441,8 +455,8 @@ void FileDetailsView::showPkgSummary( PkgInfo * pkg )
 
     if ( pkg->url() != "Pkg:/" )
     {
-        showDetails( pkg );
-        return;
+	showDetails( pkg );
+	return;
     }
 
     setCurrentPage( _ui->pkgSummaryPage );
@@ -451,29 +465,29 @@ void FileDetailsView::showPkgSummary( PkgInfo * pkg )
 
     if ( ! pkg->isBusy() && pkg->readState() == DirFinished )
     {
-        setLabel( _ui->pkgSummaryTotalSizeLabel,   pkg->totalSize()    );
-        setLabel( _ui->pkgSummaryItemCountLabel,   pkg->totalItems()   );
-        setLabel( _ui->pkgSummaryFileCountLabel,   pkg->totalFiles()   );
-        setLabel( _ui->pkgSummarySubDirCountLabel, pkg->totalSubDirs() );
+	setLabel( _ui->pkgSummaryTotalSizeLabel,   pkg->totalSize()    );
+	setLabel( _ui->pkgSummaryItemCountLabel,   pkg->totalItems()   );
+	setLabel( _ui->pkgSummaryFileCountLabel,   pkg->totalFiles()   );
+	setLabel( _ui->pkgSummarySubDirCountLabel, pkg->totalSubDirs() );
     }
     else
     {
-        QString msg;
+	QString msg;
 
-        if ( pkg->isBusy() )
-        {
-            msg = tr( "[Reading]" );
-        }
-        else
-        {
-            if ( pkg->readState() == DirError )
-                msg = tr( "[Read Error]" );
-        }
+	if ( pkg->isBusy() )
+	{
+	    msg = tr( "[Reading]" );
+	}
+	else
+	{
+            if ( pkg->readError() )
+		msg = tr( "[Read Error]" );
+	}
 
-        _ui->pkgSummaryTotalSizeLabel->setText( msg );
-        _ui->pkgSummaryItemCountLabel->clear();
-        _ui->pkgSummaryFileCountLabel->clear();
-        _ui->pkgSummarySubDirCountLabel->clear();
+	_ui->pkgSummaryTotalSizeLabel->setText( msg );
+	_ui->pkgSummaryItemCountLabel->clear();
+	_ui->pkgSummaryFileCountLabel->clear();
+	_ui->pkgSummarySubDirCountLabel->clear();
     }
 
     _ui->pkgSummaryLatestMTimeLabel->setText( formatTime( pkg->latestMtime() ) );
@@ -519,17 +533,21 @@ void FileDetailsView::showSelectionSummary( const FileInfoSet & selectedItems )
 }
 
 
-void FileDetailsView::setLabel( QLabel * label, int number )
+void FileDetailsView::setLabel( QLabel *	label,
+				int		number,
+				const QString & prefix )
 {
     CHECK_PTR( label );
-    label->setText( QString::number( number ) );
+    label->setText( prefix + QString::number( number ) );
 }
 
 
-void FileDetailsView::setLabel( QLabel * label, FileSize size )
+void FileDetailsView::setLabel( FileSizeLabel * label,
+				FileSize	size,
+				const QString & prefix )
 {
     CHECK_PTR( label );
-    label->setText( formatSize( size ) );
+    label->setValue( size, prefix );
 }
 
 
@@ -562,3 +580,36 @@ QString FileDetailsView::mimeCategory( FileInfo * file )
 
     return category ? category->name() : "";
 }
+
+
+void FileDetailsView::setLabelColor( QLabel * label, const QColor & color )
+{
+    CHECK_PTR( label );
+
+    QPalette pal = label->palette();
+    pal.setColor( QPalette::WindowText, color );
+    label->setPalette( pal );
+}
+
+
+void FileDetailsView::readSettings()
+{
+    Settings settings;
+    settings.beginGroup( "DetailsPanel" );
+
+    _dirReadErrColor = readColorEntry( settings, "DirReadErrColor", QColor( Qt::red ) );
+
+    settings.endGroup();
+}
+
+
+void FileDetailsView::writeSettings()
+{
+    Settings settings;
+    settings.beginGroup( "DetailsPanel" );
+
+    writeColorEntry( settings, "DirReadErrColor", _dirReadErrColor );
+
+    settings.endGroup();
+}
+
