@@ -109,6 +109,16 @@ namespace QDirStat
          **/
         QString mimeCategory( FileInfo * fileInfo );
 
+	/**
+	 * Read parameters from settings file.
+	 **/
+	void readSettings();
+
+	/**
+	 * Write parameters to settings file.
+	 **/
+	void writeSettings();
+
 
     protected slots:
 
@@ -126,27 +136,42 @@ namespace QDirStat
         QString parentPath( FileInfo * fileInfo );
 
         /**
-         * Set a label with a number.
+         * Set a label with a number and an optional prefix.
          **/
-        void setLabel( QLabel * label, int number );
+        void setLabel( QLabel * label, int number, const QString & prefix = "" );
 
         /**
-         * Set a label with a file size.
+         * Set a file size label with a file size and an optional prefix.
          **/
-        void setLabel( QLabel * label, FileSize size );
+        void setLabel( FileSizeLabel *  label,
+                       FileSize         size,
+                       const QString &  prefix = "" );
 
         /**
          * Set a label with a text of limited size.
          **/
         void setLabelLimited( QLabel * label, const QString & text );
 
+        /**
+         * Return 'longText' limited to a predefined maximum size.
+         **/
         QString limitText( const QString & longText );
 
         /**
-         * Format a file size suitable for this view, including handling sparse
-         * files and files with multiple hard links.
+         * Set the text of a file size label including special handling for
+         * sparse files and files with multiple hard links.
+         *
+         * Notice that this is only useful for plain files, not for
+         * directories, packages or multiple selected files.
          **/
-        QString fileSizeText( FileInfo * file ) const;
+        void setFileSizeLabel( FileSizeLabel * label,
+                               FileInfo      * file );
+
+        /**
+         * Set the text color for a label.
+         **/
+        void setLabelColor( QLabel * label, const QColor & color );
+
 
         // Boilerplate widget setting methods
 
@@ -165,6 +190,8 @@ namespace QDirStat
         Ui::FileDetailsView * _ui;
         AdaptiveTimer *       _pkgUpdateTimer;
         int                   _labelLimit;
+        QColor                _dirReadErrColor;
+        QColor                _normalTextColor;
 
     };  // class FileDetailsView
 }	// namespace QDirStat
