@@ -49,6 +49,7 @@ namespace QDirStat
 	DirOnRequestOnly,	// Will be read upon explicit request only (mount points)
 	DirCached,		// Content was read from a cache
 	DirAborted,		// Reading aborted upon user request
+        DirPermissionDenied,    // Insufficient permissions for reading
 	DirError		// Error while reading
     };
 
@@ -370,12 +371,31 @@ namespace QDirStat
 	 **/
 	virtual int directChildrenCount() { return 0; }
 
+        /**
+         * Returns the number of subdirectories below this item that could not
+         * be read (typically due to insufficient permissions).
+         *
+         * Notice that this does NOT include this item if it is a directory
+         * that could not be read.
+	 *
+	 * Derived classes that have children should overwrite this.
+         **/
+        virtual int errSubDirCount() { return 0; }
+
 	/**
 	 * Returns the latest modification time of this subtree.
 	 * Derived classes that have children should overwrite this.
 	 **/
 	virtual time_t latestMtime() { return _mtime; }
 
+	/**
+	 * Returns the oldest modification time of any file in this subtree or
+	 * 0 if there is no file.
+         *
+         * Derived classes that have children should overwrite this.
+	 **/
+        virtual time_t oldestFileMtime() { return isFile() ? _mtime : 0; }
+
 	/**
 	 * Return the percentage of this subtree in regard to its parent
 	 * (0.0..100.0). Return a negative value if for any reason this cannot
@@ -632,6 +652,26 @@ namespace QDirStat
 	 **/
 	virtual DirReadState readState() const { return DirFinished; }
 
+        /**
+         * Check if readState() is anything that indicates an error reading the
+         * directory, i.e. DirError or DirPermissionDenied.
+         *
+         * This default implementation always returns 'false'.
+	 * Derived classes should overwrite this.
+         **/
+        virtual bool readError() const { return false; }
+
+        /**
+         * Return a prefix for the total size (and similar accumulated fields)
+         * of this item: ">" if there might be more, i.e. if a subdirectory
+         * could not be read or if reading was aborted, an empty string
+         * otherwise.
+         *
+         * This default implementation returns an empty string. Derived classes
+         * that can handle child items should reimplement this.
+         **/
+        virtual QString sizePrefix() const { return ""; }
+
 	/**
 	 * Returns true if this is a DirInfo object.
 	 *
@@ -815,7 +855,20 @@ namespace QDirStat
      *
      *	   logDebug() << "Size: " << x->totalSize() << endl;
      **/
-    QString formatSize ( FileSize lSize );
+    QString formatSize ( FileSize size );
+
+    /**
+     * Format a file / subtree size as bytes, but still human readable with a
+     * space as a thousands separator, i.e. "12 345 678 Bytes".
+     *
+     * Intentionally NOT using the locale's thousands separator since this
+     * causes confusion to no end when it's only one of them, and it's unclear
+     * what locale is used: German uses "," as the decimal separator and "." as
+     * the thousands separator, exactly the other way round as English. So it's
+     * never clear if 12.345 is a little more than twelve or twelve thousand.
+     * A space character avoids this confusion.
+     **/
+    QString formatByteSize( FileSize size );
 
     /**
      * Format a timestamp (like the latestMTime()) human-readable.
