@@ -38,6 +38,7 @@
 #include "MimeCategoryConfigPage.h"
 #include "OpenPkgDialog.h"
 #include "OutputWindow.h"
+#include "PanelMessage.h"
 #include "PkgManager.h"
 #include "PkgQuery.h"
 #include "Refresher.h"
@@ -73,6 +74,7 @@ MainWindow::MainWindow():
     _ui( new Ui::MainWindow ),
     _configDialog(0),
     _modified( false ),
+    _enableDirPermissionsWarning( false ),
     _verboseSelection( false ),
     _urlInWindowTitle( false ),
     _useTreemapHover( false ),
@@ -351,12 +353,12 @@ void MainWindow::connectActions()
 
     // "Help" menu
 
-    CONNECT_ACTION( _ui->actionHelp,		this, showHelp()          );
-    CONNECT_ACTION( _ui->actionPkgViewHelp,	this, showPkgViewHelp()   );
+    CONNECT_ACTION( _ui->actionHelp,		this, showHelp()	  );
+    CONNECT_ACTION( _ui->actionPkgViewHelp,	this, showPkgViewHelp()	  );
     CONNECT_ACTION( _ui->actionUnpkgViewHelp,	this, showUnpkgViewHelp() );
-    CONNECT_ACTION( _ui->actionAbout,		this, showAboutDialog()   );
-    CONNECT_ACTION( _ui->actionAboutQt,		qApp, aboutQt()           );
-    CONNECT_ACTION( _ui->actionWhatsNew,	this, showWhatsNew()      );
+    CONNECT_ACTION( _ui->actionAbout,		this, showAboutDialog()	  );
+    CONNECT_ACTION( _ui->actionAboutQt,		qApp, aboutQt()		  );
+    CONNECT_ACTION( _ui->actionWhatsNew,	this, showWhatsNew()	  );
     CONNECT_ACTION( _ui->actionDonate,		this, showDonateDialog()  );
 
 
@@ -593,6 +595,20 @@ void MainWindow::busyDisplay()
 {
     _ui->treemapView->disable();
     updateActions();
+
+    if ( _unreadableDirsWindow )
+    {
+	// Close the window that lists unreadable directories: With the next
+	// directory read, things might have changed; the user may have fixed
+	// permissions or ownership of those directories.
+	//
+	// Closing this window also deletes it (because it has the
+	// DeleteOnClose flag set). The QPointer we use will take care of
+	// resetting itself to 0 when the underlying QObject is deleted.
+
+	_unreadableDirsWindow->close();
+    }
+
     _updateTimer.start();
 
     // It would be nice to sort by read jobs during reading, but this confuses
@@ -646,6 +662,12 @@ void MainWindow::readingFinished()
     _ui->statusBar->showMessage( tr( "Finished. Elapsed time: %1").arg( elapsedTime ), LONG_MESSAGE );
     logInfo() << "Reading finished after " << elapsedTime << endl;
 
+    if ( _dirTreeModel->tree()->firstToplevel() &&
+	 _dirTreeModel->tree()->firstToplevel()->errSubDirCount() > 0 )
+    {
+	showDirPermissionsWarning();
+    }
+
     // Debug::dumpModelTree( _dirTreeModel, QModelIndex(), "" );
 }
 
@@ -663,6 +685,8 @@ void MainWindow::readingAborted()
 
 void MainWindow::openUrl( const QString & url )
 {
+    _enableDirPermissionsWarning = true;
+
     if ( PkgFilter::isPkgUrl( url ) )
 	readPkg( url );
     else if ( isUnpkgUrl( url ) )
@@ -831,6 +855,7 @@ bool MainWindow::isUnpkgUrl( const QString & url )
 
 void MainWindow::refreshAll()
 {
+    _enableDirPermissionsWarning = true;
     QString url = _dirTreeModel->tree()->url();
 
     if ( ! url.isEmpty() )
@@ -956,9 +981,17 @@ void MainWindow::showCurrent( FileInfo * item )
 {
     if ( item )
     {
-	_ui->statusBar->showMessage( QString( "%1  (%2)" )
-				     .arg( item->debugUrl() )
-				     .arg( formatSize( item->totalSize() ) ) );
+	QString msg = QString( "%1  (%2%3)" )
+	    .arg( item->debugUrl() )
+	    .arg( item->sizePrefix() )
+	    .arg( formatSize( item->totalSize() ) );
+
+	if ( item->readState() == DirPermissionDenied )
+	    msg += tr( "  [Permission Denied]" );
+        else if ( item->readState() == DirError )
+	    msg += tr( "  [Read Error]" );
+
+	_ui->statusBar->showMessage( msg );
     }
     else
     {
@@ -1150,8 +1183,7 @@ void MainWindow::showFileTypeStats()
 	// This deletes itself when the user closes it. The associated QPointer
 	// keeps track of that and sets the pointer to 0 when it happens.
 
-	_fileTypeStatsWindow = new QDirStat::FileTypeStatsWindow( _selectionModel,
-								  this );
+	_fileTypeStatsWindow = new FileTypeStatsWindow( _selectionModel, this );
     }
 
     _fileTypeStatsWindow->populate( selectedDirOrRoot() );
@@ -1270,6 +1302,41 @@ FileInfo * MainWindow::selectedDirOrRoot() const
 }
 
 
+void MainWindow::showDirPermissionsWarning()
+{
+    if ( _dirPermissionsWarning || ! _enableDirPermissionsWarning )
+	return;
+
+    PanelMessage * msg = new PanelMessage( _ui->messagePanel );
+    CHECK_NEW( msg );
+
+    msg->setHeading( tr( "Some directories could not be read." ) );
+    msg->setText( tr( "You might not have sufficient permissions." ) );
+    msg->setIcon( QPixmap( ":/icons/lock-closed.png" ) );
+
+    msg->connectDetailsLink( this, SLOT( showUnreadableDirs() ) );
+
+    _ui->messagePanel->add( msg );
+    _dirPermissionsWarning = msg;
+    _enableDirPermissionsWarning = false;
+}
+
+
+void MainWindow::showUnreadableDirs()
+{
+    if ( ! _unreadableDirsWindow )
+    {
+	// This deletes itself when the user closes it. The associated QPointer
+	// keeps track of that and sets the pointer to 0 when it happens.
+
+	_unreadableDirsWindow = new UnreadableDirsWindow( _selectionModel, this );
+    }
+
+    _unreadableDirsWindow->populate( _dirTreeModel->tree()->root() );
+    _unreadableDirsWindow->show();
+}
+
+
 void MainWindow::toggleVerboseSelection()
 {
     // Verbose selection is toggled with Shift-F7
@@ -1318,7 +1385,7 @@ void MainWindow::showAboutDialog()
     text += tr( "Qt-based directory statistics -- showing where all your disk space has gone "
 		" and trying to help you to clean it up." );
     text += "</p><p>";
-    text += "(c) 2015-2019 Stefan Hundhammer";
+    text += "(c) 2015-2020 Stefan Hundhammer";
     text += "</p><p>";
     text += tr( "Contact: " ) + QString( "<a href=\"mailto:%1\">%2</a>" ).arg( mailTo ).arg( mailTo );
     text += "</p><p>";
@@ -1440,17 +1507,24 @@ QString MainWindow::formatTime( qint64 millisec, bool showMillisec )
 
     if ( hours < 1 && min < 1 && sec < 60 )
     {
-	if ( showMillisec )
-	    formattedTime.sprintf ( "%2d.%03lld ", sec, millisec );
-	else
-	    formattedTime.sprintf ( "%2d ", sec );
-	    // formattedTime.sprintf ( "%2d.%1lld ", sec, millisec / 100 );
+	formattedTime.setNum( sec );
 
-	formattedTime += tr( "sec" );
+	if ( showMillisec )
+	{
+	    formattedTime += QString( ".%1" ).arg( millisec,
+						   3,	// fieldWidth
+						   10,	// base
+						   QChar( '0' ) ); // fillChar
+	}
+
+	formattedTime += " " + tr( "sec" );
     }
     else
     {
-	formattedTime.sprintf ( "%02d:%02d:%02d", hours, min, sec );
+	formattedTime = QString( "%1:%2:%3" )
+	    .arg( hours, 2, 10, QChar( '0' ) )
+	    .arg( min,	 2, 10, QChar( '0' ) )
+	    .arg( sec,	 2, 10, QChar( '0' ) );
     }
 
     return formattedTime;
