@@ -18,6 +18,8 @@
 
 #include "ui_main-window.h"
 #include "FileTypeStatsWindow.h"
+#include "PanelMessage.h"
+#include "UnreadableDirsWindow.h"
 #include "PkgFilter.h"
 
 
@@ -39,6 +41,8 @@ namespace QDirStat
 
 using QDirStat::FileInfo;
 using QDirStat::FileTypeStatsWindow;
+using QDirStat::PanelMessage;
+using QDirStat::UnreadableDirsWindow;
 
 
 class MainWindow: public QMainWindow
@@ -321,6 +325,18 @@ protected slots:
      **/
     void showElapsedTime();
 
+    /**
+     * Show a warning (as a panel message) about insufficient permissions when
+     * reading directories.
+     **/
+    void showDirPermissionsWarning();
+
+    /**
+     * Show the directories that could not be read in a separate non-modal
+     * window.
+     **/
+    void showUnreadableDirs();
+
     /**
      * Switch verbose logging for selection changes on or off.
      *
@@ -418,25 +434,28 @@ protected:
 
 private:
 
-    Ui::MainWindow		* _ui;
-    QDirStat::DirTreeModel	* _dirTreeModel;
-    QDirStat::SelectionModel	* _selectionModel;
-    QDirStat::CleanupCollection * _cleanupCollection;
-    QDirStat::ConfigDialog	* _configDialog;
-    QActionGroup		* _layoutActionGroup;
-    QPointer<FileTypeStatsWindow> _fileTypeStatsWindow;
-    QString			  _dUrl;
-    QElapsedTimer		  _stopWatch;
-    bool			  _modified;
-    bool			  _verboseSelection;
-    bool			  _urlInWindowTitle;
-    bool			  _useTreemapHover;
-    QString			  _layoutName;
-    int				  _statusBarTimeout; // millisec
-    QSignalMapper	       *  _treeLevelMapper;
-    QMap<QString, TreeLayout *>	  _layouts;
-    TreeLayout *		  _currentLayout;
-    QTimer			  _updateTimer;
+    Ui::MainWindow		*  _ui;
+    QDirStat::DirTreeModel	*  _dirTreeModel;
+    QDirStat::SelectionModel	*  _selectionModel;
+    QDirStat::CleanupCollection *  _cleanupCollection;
+    QDirStat::ConfigDialog	*  _configDialog;
+    QActionGroup		*  _layoutActionGroup;
+    QPointer<FileTypeStatsWindow>  _fileTypeStatsWindow;
+    QPointer<PanelMessage>	   _dirPermissionsWarning;
+    QPointer<UnreadableDirsWindow> _unreadableDirsWindow;
+    QString			   _dUrl;
+    QElapsedTimer		   _stopWatch;
+    bool			   _modified;
+    bool			   _enableDirPermissionsWarning;
+    bool			   _verboseSelection;
+    bool			   _urlInWindowTitle;
+    bool			   _useTreemapHover;
+    QString			   _layoutName;
+    int				   _statusBarTimeout; // millisec
+    QSignalMapper	       *   _treeLevelMapper;
+    QMap<QString, TreeLayout *>	   _layouts;
+    TreeLayout *		   _currentLayout;
+    QTimer			   _updateTimer;
 
 }; // class MainWindow
 
