@@ -211,7 +211,7 @@ void SelectionModel::setCurrentItem( FileInfo * item, bool select )
 void SelectionModel::setCurrentItem( const QString & path )
 {
     FileInfo * item = _dirTreeModel->tree()->locate( path,
-                                                     true ); // findDotEntries
+						     true ); // findPseudoDirs
     if ( item )
 	setCurrentItem( item, true );
     else
@@ -238,7 +238,7 @@ void SelectionModel::prepareRefresh( const FileInfoSet & refreshSet )
     {
 	dir = current->isDirInfo() ? current->toDirInfo() : current->parent();
 
-	if ( dir && dir->isDotEntry() )
+	if ( dir && dir->isPseudoDir() )
 	    dir = dir->parent();
 
 	// Go one directory up from the current item as long as there is an
