@@ -12,6 +12,7 @@
 #include <QApplication>
 #include "MainWindow.h"
 #include "DirTreeModel.h"
+#include "PkgFilter.h"
 #include "Settings.h"
 #include "Logger.h"
 #include "Exception.h"
@@ -29,8 +30,21 @@ void usage( const QStringList & argList )
 	 << "Usage: \n"
 	 << "\n"
 	 << "  " << progName << " [--slow-update|-s] [<directory-name>]\n"
+	 << "  " << progName << " pkg:/pkgpattern\n"
+	 << "  " << progName << " unpkg:/dir\n"
+	 << "  " << progName << " --dont-ask|-d\n"
 	 << "  " << progName << " --cache|-c <cache-file-name>\n"
 	 << "  " << progName << " --help|-h\n"
+	 << "\n"
+	 << "\n"
+         << "Supported pkg patterns:\n"
+	 << "\n"
+         << "- Default: \"Starts with\" \"pkg:/mypkg\"\n"
+         << "- Wildcards with '*' and '?'\n"
+         << "- Full regexps with \".*\" and/or \"^\" and/or \"$\"\n"
+         << "- Exact match: \"pkg:/=mypkg\"\n"
+         << "- All packages: \"pkg:/\"\n"
+	 << "\n"
 	 << std::endl;
 
     logError() << "FATAL: Bad command line args: " << argList.join( " " ) << endl;
@@ -94,11 +108,16 @@ int main( int argc, char *argv[] )
     CHECK_PTR( mainWin );
     mainWin->show();
 
+    bool dont_ask = commandLineSwitch( "--dont-ask", "-d", argList );
+
     if ( commandLineSwitch( "--slow-update", "-s", argList ) )
 	mainWin->dirTreeModel()->setSlowUpdate();
 
     if ( argList.isEmpty() )
-	mainWin->askOpenUrl();
+    {
+        if ( ! dont_ask )
+            mainWin->askOpenDir();
+    }
     else
     {
 	QString arg = argList.first();
@@ -120,7 +139,7 @@ int main( int argc, char *argv[] )
 	    usage( argList );
 	else if ( ! arg.isEmpty() )
 	{
-	    mainWin->openUrl( arg );
+            mainWin->openUrl( arg );
 	}
     }
 
